package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import ch.zhaw.projekt2.turnierverwaltung.main.tournamentList.TournamentListController;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;

import java.io.IOException;
import java.net.URL;
import java.nio.charset.Charset;

public class Factory {
    private TournamentDecorator tournamentDecorator;
    private FileIO fileIO;
    //TODO save views instead of recreate them.

    public Factory(FileIO fileIO, TournamentDecorator tournamentDecorator) {
        this.fileIO = fileIO;
        this.tournamentDecorator = tournamentDecorator;
    }

    public TournamentDecorator getTournamentDecorator() {
        return tournamentDecorator;
    }

    public void setTournament(Tournament tournament) {
        this.tournamentDecorator.setTournament(tournament);
    }

    public BorderPane loadMainWindow() {
        FXMLLoader loader = new FXMLLoader(getClass().getResource("mainWindow.fxml"));
        try {
            return loader.load();
        } catch (IOException e) {
            e.printStackTrace();
            //TODO handle and logging
        }
        return null;
    }

    public void loadTournamentList(BorderPane pane, FactoryDecorator factoryDecorator) {
        tournamentDecorator.setTournament(null);
        TournamentListController controller = (TournamentListController) setCenterOfBorderPane(pane, getClass().getResource("tournamentList/tournamentList.fxml"), factoryDecorator);
    }

    //Can be used to Open new Scene in same Stage.
    //This way possible to later give object to Controller
    public void loadParticipantFormular(BorderPane pane, FactoryDecorator factoryDecorator) {
        setCenterOfBorderPane(pane, getClass().getResource("participantAddFormular/participantFormular.fxml"), factoryDecorator);
    }

    public void loadPlacesFormular(BorderPane pane, FactoryDecorator factoryDecorator) {
        setCenterOfBorderPane(pane, getClass().getResource("placesAddFormular/PlacesFormular.fxml"), factoryDecorator);
    }

    public void loadGameScheduler(BorderPane pane, FactoryDecorator factoryDecorator) {
        setCenterOfBorderPane(pane, getClass().getResource("gameScheduleView/GameSchedule.fxml"), factoryDecorator);
    }

    public GameController loadGameView(VBox box, GameDecorator gameDecorator, FactoryDecorator factoryDecorator) {
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("gameScheduleView/Game.fxml"));
            box.getChildren().add(loader.load());
            GameController controller = loader.getController();
            controller.setup(tournamentDecorator, fileIO, factoryDecorator, box, gameDecorator);
            return controller;
        } catch (IOException e) {
            e.printStackTrace();
            //TODO LOGGER
        }
        return null;
    }

    private FXController setCenterOfBorderPane(BorderPane pane, URL location, FactoryDecorator factoryDecorator) {
        FXController controller = null;
        try {
            FXMLLoader loader = new FXMLLoader(location);
            pane.setCenter(loader.load());
            controller = loader.getController();
            controller.setup(tournamentDecorator, fileIO, factoryDecorator, pane);
            VBox bottom = (VBox) pane.getBottom();
            resetFooter(pane, true);
        } catch (IOException e) {
            e.printStackTrace();
            //TODO handle and logging?
        }
        return controller;
    }


    public void printMessageToFooter(BorderPane pane, String msg, boolean error) {
        VBox bottom = (VBox) pane.getBottom();
        Label label = new Label();
        VBox innerVbox;
        label.setText(msg);

        if (error) {
            innerVbox = (VBox) bottom.getChildren().get(0);
            resetFooter(pane, true);
            label.setTextFill(Color.RED);
            label.setFont(new Font(20));
            innerVbox.setBorder(new Border(new BorderStroke(Color.valueOf("#FF0000"), BorderStrokeStyle.SOLID, CornerRadii.EMPTY, BorderWidths.DEFAULT)));
            innerVbox.getChildren().add(label);

        } else {
            innerVbox = (VBox) bottom.getChildren().get(1);
            resetFooter(pane,false);
            label.setTextFill(Color.GREEN);
            label.setFont(new Font(10));
            label.setPadding(new Insets(0,10,0,0));
            innerVbox.getChildren().add(label);

        }


    }

    public void resetFooter(BorderPane pane,boolean error) {
        VBox bottom = (VBox) pane.getBottom();
        VBox vBox;
        if (error) {
            vBox = (VBox) bottom.getChildren().get(0);
        } else {
            vBox = (VBox) bottom.getChildren().get(1);
        }
        vBox.getChildren().clear();
        vBox.setBorder(null);
    }

}
